/************************************************************************
 *   IRC - Internet Relay Chat, doc/example_module.c
 *   Copyright (C) 2001 Hybrid Development Team
 *   Copyright (C) 2003-2004 BgChat Develops HaKer4o
 * 
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   $Id: m_svsid.c,v 1.1 2004/06/15 04:28:17 HaKer4o Exp $
 */

/* List of ircd includes from ../include/ */
#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "common.h"     /* FALSE bleah */
#include "ircd.h"
#include "irc_string.h"
#include "numeric.h"
#include "fdlist.h"
#include "s_bsd.h"
#include "s_conf.h"
#include "s_log.h"
#include "s_serv.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"

#include "s_user.h"  /* send_umode_out() */
#include "channel_mode.h" /* MODE_ADD & MOD_DEL */
#include "hash.h"


static void m_svsmode	(struct Client *, struct Client *, int, char **);
static void m_svsident	(struct Client *, struct Client *, int, char **);


struct Message setmode_msgtab = {
  "SETMODE", 0, 0, 2, 3, MFLG_SLOW, 0,
  {m_ignore, m_not_oper, m_ignore, m_ignore, m_setmode}
};

struct Message setident_msgtab = {
  "SETIDENT", 0, 0, 2, 3, MFLG_SLOW, 0,
  {m_ignore, m_not_oper, m_ignore, m_ignore, m_svsident  }
};

#ifndef STATIC_MODULES
void _modinit (void)

mod_add_cmd (&setmode_msgtab);
mod_add_cmd (&setident_msgtab);

void _moddeinit (void)
{

mod_del_cmd (&setmode_msgtab);
mod_del_cmd (&setident_msgtab);

const char *_version = "$Revision: 1.1 $";
#endif

static void mo_svsmode (struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	struct Client *target_p;
	char target[NICKLEN], **p, *m;
	int flag, i, what, setflags;

	if (ConfigFileEntry.disable_svs_commands)
		return;

	strlcpy (target, parv[1], NICKLEN);

	what = MODE_ADD;

	if ((target_p = find_client (target)) != NULL)
	{
		setflags = target_p->umodes;

		for (p = &parv[2]; p && *p; p++)
		{
			for (m = *p; *m; m++)
			{
				switch (*m)
				{
					case '+':
					what = MODE_ADD;
						break;
					case '-':
						what = MODE_DEL;
						break;
					case 'o':
						break;
					case ' ':
					case '\n':
					case '\r':
					case '\t':
						break;

					default:
						if ((flag = user_modes_from_c_to_bitmask[(unsigned char)*m]))
						{
							if ((ConfigFileEntry.oper_only_umodes & flag) && 
(!IsOper(target_p)))
								;
							else
							{
								if (what == MODE_ADD)
									target_p->umodes |= flag;
								else
									target_p->umodes &= ~flag;
							}
						}
					break;
				}
			}
		}

		send_umode_out (target_p, target_p, setflags);
        sendto_realops_flags (UMODE_SPY, L_ALL, "SVSMODE called for %s[%s] (%s)", target, parv[2], 
get_oper_name(source_p));
        sendto_server (client_p, NULL, CAP_ENCAP, NOCAPS, ":%s ENCAP * OPERSPY SVSMODE called for %s[%s]", 
source_p->name, target, parv[2]);
		return;
	}

	sendto_one(source_p, form_str(ERR_NOSUCHNICK), me.name, source_p->name, target);
	return;
}

static void m_svsident (struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	struct Client *target_p;
	char target[NICKLEN], username[USERLEN];

	if (ConfigFileEntry.disable_svs_commands)
		return;

	strlcpy (target, parv[1], NICKLEN);
	strlcpy (username, parv[2], USERLEN);

	if ((target_p = find_client (target)) != NULL)
	{
		if (!clean_user_name(username))
		{
			sendto_one (source_p, ":%s NOTICE %s :Invalid username", me.name, source_p->name);
			return;
		}

		if (!strcmp (target_p->username, username))
			return;

		strlcpy (target_p->username, username, USERLEN);
		sendto_server (NULL, NULL, NOCAPS, NOCAPS, ":%s SVSIDENT %s %s", me.name, target, username);
		return;
	}

	sendto_one(source_p, form_str(ERR_NOSUCHNICK), me.name, source_p->name, target);
	return;
}


